@@ -150,7 +150,7 @@ gst_faac_profile_get_type (void)
   if (!gst_faac_profile_type) {
     static GEnumValue gst_faac_profile[] = {
       {MAIN, "MAIN", "Main profile"},
-      {LOW, "LOW", "Low complexity profile"},
+      {LOW, "LC", "Low complexity profile"},
       {SSR, "SSR", "Scalable sampling rate profile"},
       {LTP, "LTP", "Long term prediction profile"},
       {0, NULL, NULL},
@@ -218,7 +218,7 @@ gst_faac_class_init (GstFaacClass * klass)
   /* properties */
   g_object_class_install_property (gobject_class, ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate (bps)", "Bitrate in bits/sec",
-          8 * 1024, 320 * 1024, 128 * 1024, G_PARAM_READWRITE));
+          8 * 1000, 320 * 1000, 128 * 1000, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PROFILE,
       g_param_spec_enum ("profile", "Profile", "MPEG/AAC encoding profile",
           GST_TYPE_FAAC_PROFILE, MAIN, G_PARAM_READWRITE));
@@ -251,9 +251,7 @@ gst_faac_init (GstFaac * faac)
   faac->cache_time = GST_CLOCK_TIME_NONE;
   faac->cache_duration = 0;
 
-  faac->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
-      "sink");
+  faac->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
   gst_pad_set_chain_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_chain));
   gst_pad_set_setcaps_function (faac->sinkpad,
@@ -262,14 +260,12 @@ gst_faac_init (GstFaac * faac)
       GST_DEBUG_FUNCPTR (gst_faac_sink_event));
   gst_element_add_pad (GST_ELEMENT (faac), faac->sinkpad);
 
-  faac->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
-      "src");
+  faac->srcpad = gst_pad_new_from_static_template (&src_template, "src");
   gst_pad_use_fixed_caps (faac->srcpad);
   gst_element_add_pad (GST_ELEMENT (faac), faac->srcpad);
 
   /* default properties */
-  faac->bitrate = 1024 * 128;
+  faac->bitrate = 1000 * 128;
   faac->profile = MAIN;
   faac->shortctl = SHORTCTL_NORMAL;
   faac->outputformat = 0;
@@ -457,9 +453,13 @@ gst_faac_sink_event (GstPad * pad, GstEvent * event)
     {
       GstBuffer *outbuf;
 
+      if (!faac->handle)
+        ret = FALSE;
+      else
+        ret = TRUE;
+
       /* flush first */
-      ret = TRUE;
-      do {
+      while (ret) {
         if (gst_pad_alloc_buffer_and_set_caps (faac->srcpad,
                 GST_BUFFER_OFFSET_NONE, faac->bytes,
                 GST_PAD_CAPS (faac->srcpad), &outbuf) == GST_FLOW_OK) {
@@ -476,8 +476,7 @@ gst_faac_sink_event (GstPad * pad, GstEvent * event)
             ret = FALSE;
           }
         }
-      } while (ret);
-
+      }
       ret = gst_pad_event_default (pad, event);
       break;
     }
