@@ -31,7 +31,8 @@ plugin_init (GstPlugin * plugin)
 {
   if (!gst_element_register (plugin, "gsmenc", GST_RANK_NONE, GST_TYPE_GSMENC))
     return FALSE;
-  if (!gst_element_register (plugin, "gsmdec", GST_RANK_NONE, GST_TYPE_GSMDEC))
+  if (!gst_element_register (plugin, "gsmdec", GST_RANK_PRIMARY,
+          GST_TYPE_GSMDEC))
     return FALSE;
 
   return TRUE;
