@@ -19,15 +19,14 @@
 #endif
 
 #include "gstneonhttpsrc.h"
+#include <stdlib.h>
 #include <string.h>
+#ifdef _HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* _HAVE_UNISTD_H */
 
 #include <ne_redirect.h>
 
-#ifndef NE_FREE
-#define NEON_026_OR_LATER  1
-#endif
-
 GST_DEBUG_CATEGORY_STATIC (neonhttpsrc_debug);
 #define GST_CAT_DEFAULT neonhttpsrc_debug
 
@@ -56,7 +55,7 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 #define HTTP_DEFAULT_HOST        "localhost"
 
 /* default properties */
-#define DEFAULT_LOCATION             "http://"HTTP_DEFAULT_HOST"/"G_STRINGIFY(HTTP_DEFAULT_PORT)
+#define DEFAULT_LOCATION             "http://"HTTP_DEFAULT_HOST":"G_STRINGIFY(HTTP_DEFAULT_PORT)
 #define DEFAULT_PROXY                ""
 #define DEFAULT_USER_AGENT           "GStreamer neonhttpsrc"
 #define DEFAULT_IRADIO_MODE          FALSE
@@ -286,6 +285,9 @@ gst_neonhttp_src_dispose (GObject * gobject)
   if (src->location) {
     ne_free (src->location);
   }
+  if (src->query_string) {
+    ne_free (src->query_string);
+  }
 
   G_OBJECT_CLASS (parent_class)->dispose (gobject);
 }
@@ -705,6 +707,10 @@ gst_neonhttp_src_set_location (GstNeonhttpSrc * src, const gchar * uri)
     ne_free (src->location);
     src->location = NULL;
   }
+  if (src->query_string) {
+    ne_free (src->query_string);
+    src->query_string = NULL;
+  }
 
   if (ne_uri_parse (uri, &src->uri) != 0)
     goto parse_error;
@@ -725,6 +731,8 @@ gst_neonhttp_src_set_location (GstNeonhttpSrc * src, const gchar * uri)
   if (!src->uri.path)
     src->uri.path = g_strdup ("");
 
+  src->query_string = g_strjoin ("?", src->uri.path, src->uri.query, NULL);
+
   src->location = ne_uri_unparse (&src->uri);
 
   return TRUE;
@@ -736,6 +744,10 @@ parse_error:
       ne_free (src->location);
       src->location = NULL;
     }
+    if (src->query_string) {
+      ne_free (src->query_string);
+      src->query_string = NULL;
+    }
     ne_uri_free (&src->uri);
     return FALSE;
   }
@@ -756,13 +768,8 @@ gst_neonhttp_src_set_proxy (GstNeonhttpSrc * src, const char *uri)
   if (src->proxy.host && !src->proxy.port)
     goto error;
 
-#ifdef NEON_026_OR_LATER
   if (!src->proxy.path || src->proxy.userinfo)
     goto error;
-#else
-  if (!src->proxy.path || src->proxy.authinfo)
-    goto error;
-#endif
   return TRUE;
 
   /* ERRORS */
@@ -799,7 +806,9 @@ gst_neonhttp_src_send_request_and_redirect (GstNeonhttpSrc * src,
           ne_session_create (src->uri.scheme, src->uri.host, src->uri.port);
     }
 
-    request = ne_request_create (session, "GET", src->uri.path);
+    ne_set_session_flag (session, NE_SESSFLAG_ICYPROTO, 1);
+
+    request = ne_request_create (session, "GET", src->query_string);
 
     if (src->user_agent) {
       ne_add_request_header (request, "User-Agent", src->user_agent);
