@@ -53,8 +53,8 @@ static const GstElementDetails gst_sdlvideosink_details =
 GST_ELEMENT_DETAILS ("SDL video sink",
     "Sink/Video",
     "An SDL-based videosink",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>"
-    "Edgard Lima <edgard.lima@indt.org.br>"
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>\n"
+    "Edgard Lima <edgard.lima@indt.org.br>\n"
     "Jan Schmidt <thaytan@mad.scientist.com>");
 
 
@@ -127,9 +127,10 @@ _do_init (GType type)
 }
 
 GST_BOILERPLATE_FULL (GstSDLVideoSink, gst_sdlvideosink, GstVideoSink,
-    GST_TYPE_VIDEO_SINK, _do_init)
+    GST_TYPE_VIDEO_SINK, _do_init);
 
-     static void gst_sdlvideosink_base_init (gpointer g_class)
+static void
+gst_sdlvideosink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
   GstCaps *capslist;
@@ -337,7 +338,7 @@ gst_sdlvideosink_supported (GstImplementsInterface * interface,
 
       /* True if the video driver is X11 */
       result = (strcmp ("x11", SDL_VideoDriverName (tmp, 4)) == 0);
-      SDL_Quit ();
+      SDL_QuitSubSystem (SDL_INIT_VIDEO);
       g_mutex_unlock (sdlvideosink->lock);
     } else
       result = sdlvideosink->is_xwindows;
@@ -483,7 +484,7 @@ gst_sdlvideosink_deinitsdl (GstSDLVideoSink * sdlvideosink)
       sdlvideosink->event_thread = NULL;
     }
 
-    SDL_Quit ();
+    SDL_QuitSubSystem (SDL_INIT_VIDEO);
     sdlvideosink->init = FALSE;
 
   }
@@ -495,7 +496,7 @@ gst_sdlv_process_events (GstSDLVideoSink * sdlvideosink)
 {
   SDL_Event event;
   int numevents;
-  char *keysym;
+  char *keysym = NULL;
 
   do {
     SDL_PumpEvents ();
@@ -716,7 +717,6 @@ static gboolean
 gst_sdlvideosink_setcaps (GstBaseSink * bsink, GstCaps * vscapslist)
 {
   GstSDLVideoSink *sdlvideosink;
-  guint32 format;
   GstStructure *structure;
   gboolean res = TRUE;
 
@@ -745,7 +745,6 @@ gst_sdlvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 {
 
   GstSDLVideoSink *sdlvideosink;
-  SDL_Event sdl_event;
 
   sdlvideosink = GST_SDLVIDEOSINK (bsink);
 
@@ -779,7 +778,9 @@ gst_sdlvideosink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
           v = y + I420_V_OFFSET (sdlvideosink->width, sdlvideosink->height);
           break;
         default:
-          g_assert_not_reached ();
+          gst_sdlvideosink_unlock (sdlvideosink);
+          g_mutex_unlock (sdlvideosink->lock);
+          g_return_val_if_reached (GST_FLOW_ERROR);
       }
 
       /* Y Plane */
@@ -947,7 +948,6 @@ gst_sdlvideosink_navigation_send_event (GstNavigation * navigation,
   GstSDLVideoSink *sdlvideosink = GST_SDLVIDEOSINK (navigation);
   GstEvent *event;
   GstVideoRectangle src, dst, result;
-  gint width, height;
   double x, y;
   GstPad *pad = NULL;
 
