@@ -64,12 +64,7 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 
-/* XvidEnc signals and properties */
-enum
-{
-  FRAME_ENCODED,
-  LAST_SIGNAL
-};
+/* XvidEnc properties */
 
 /* maximum property-id */
 static int xvidenc_prop_count;
@@ -98,7 +93,6 @@ static GstStateChangeReturn gst_xvidenc_change_state (GstElement * element,
     GstStateChange transition);
 
 static GstElementClass *parent_class = NULL;
-static guint gst_xvidenc_signals[LAST_SIGNAL] = { 0 };
 
 #define GST_TYPE_XVIDENC_PROFILE (gst_xvidenc_profile_get_type ())
 static GType
@@ -457,13 +451,6 @@ gst_xvidenc_class_init (GstXvidEncClass * klass)
       G_PARAM_READWRITE);
   gst_xvidenc_add_pspec (gobject_class, pspec, container_frame_overhead);
 
-  /* signals */
-  gst_xvidenc_signals[FRAME_ENCODED] =
-      g_signal_new ("frame-encoded", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstXvidEncClass, frame_encoded),
-      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
-
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_xvidenc_change_state);
 }
 
@@ -475,9 +462,7 @@ gst_xvidenc_init (GstXvidEnc * xvidenc)
   guint i, num_props;
 
   /* create the sink pad */
-  xvidenc->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
-      "sink");
+  xvidenc->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
   gst_element_add_pad (GST_ELEMENT (xvidenc), xvidenc->sinkpad);
 
   gst_pad_set_chain_function (xvidenc->sinkpad,
@@ -488,9 +473,7 @@ gst_xvidenc_init (GstXvidEnc * xvidenc)
       GST_DEBUG_FUNCPTR (gst_xvidenc_handle_sink_event));
 
   /* create the src pad */
-  xvidenc->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
-      "src");
+  xvidenc->srcpad = gst_pad_new_from_static_template (&src_template, "src");
   gst_element_add_pad (GST_ELEMENT (xvidenc), xvidenc->srcpad);
   gst_pad_use_fixed_caps (xvidenc->srcpad);
 
@@ -513,6 +496,7 @@ gst_xvidenc_init (GstXvidEnc * xvidenc)
     g_param_value_set_default (pspec, &val);
     g_object_set_property (G_OBJECT (xvidenc), g_param_spec_get_name (pspec),
         &val);
+    g_value_unset (&val);
   }
 
   g_free (pspecs);
@@ -535,6 +519,8 @@ gst_xvidenc_finalize (GObject * object)
   GstXvidEnc *xvidenc = GST_XVIDENC (object);
 
   g_free (xvidenc->filename);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
@@ -966,9 +952,6 @@ gst_xvidenc_chain (GstPad * pad, GstBuffer * buf)
   if (!outbuf)                  /* error or no data yet */
     return GST_FLOW_OK;
 
-  /* proclaim destiny */
-  g_signal_emit (G_OBJECT (xvidenc), gst_xvidenc_signals[FRAME_ENCODED], 0);
-
   /* go out, multiply! */
   return gst_pad_push (xvidenc->srcpad, outbuf);
 }
